<?php declare(strict_types=1);

namespace Lewisdale\Webmentions;

use League\Uri\Exceptions\SyntaxError;
use League\Uri\Uri;

class Url
{
    public static function matchesHost(string $source, string $host): bool
    {
        $uri = Uri::createFromString($source);
        return $uri->getHost() === $host;
    }

    public static function validateUrl(string $url): bool
    {
        try {
            $uri = Uri::createFromString($url);
            $scheme = $uri->getScheme();
            $schemeValid = in_array($scheme, ["http", "https"]);
            [, $tld] = explode(".", $uri->getHost() ?? "");

            return !empty($tld) && $schemeValid && !!filter_var($url, FILTER_VALIDATE_URL);

        } catch (SyntaxError) {
            return false;
        }
    }
}